//
// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) 2020 - Stephane MOTTELET
//
// This file is hereby licensed under the terms of the GNU GPL v2.0,
// For more information, see the COPYING file which you should have received
//
//--------------------------------------------------------------------------

function testHessian()
    function y=g(x)
        y = A*x;
    end
    
    //grand("setsd",23)
    N = 50;
    A = sprand(N,N,0.05);
    A=A+A';
    x0 = rand(N,1);
    
    //H1 = sparse(numderivative(g,x0));
    sp = A<>0;
    
    // estimate Hessian using old implementation (without objects)
    //[H2,seed,prod]=numsphessian(g,x0,sp);
    
    // get computation engine
    params = struct("Coloring","STAR","FiniteDifferenceType","COMPLEXSTEP","Vectorized","on");
    hessian = spCompHessian(g,sp,params);
    
    // compute Hessian with new implementation
    H3 = hessian(x0)
    clf;
    subplot(1,2,1)
    spycol((A),hessian.colors)
    title("Colored Hessian pattern")
    isoview on
    subplot(1,2,2)
    spycol(hessian.seed,0:size(hessian.seed,2)-1)
    title("Seed matrix")
endfunction

testHessian()
clear testhessian
